// This class encapsulates a chat entity, which can be a Room, Person, or Note. 

import java.util.*;
import javax.swing.text.*;

public abstract class Entity implements Comparable {

  protected String newline = "\n";
  protected String tab = "\t";
  
  // For a Room it's the venue name; for a Person it's the nick; for a note it's the sender 
  protected String name;

  // The description of this entity; for a Note, it is the subject.
  protected String description;

  /* Name of an organization; can be used as a namespace or for sorting.  For a Room, it's
     the VO name; for a Person, it's the abbreviated form of the "O" field of an X.509 cert.
  */
  protected String organization = null; 

  /* Indicates whether this entity is currently active.
     For a Person, it indicates whether s/he is online or not. 
  */
  protected boolean active = false;

  // Indicate whether this entity is on a Favorites list.
  protected boolean favorite = false;  

  // Indicates availability: for a Person: "available", "busy", "away".
  protected String avail = "available";

  /* A set containing the current contents or locations of this entity.  For a room, 
     it's the set of people or other objects in the room.  For a person, it's the set 
     of rooms s/he is in.  For a note, it's a set of recipients.	
  */
  protected Set info = new HashSet();

  public Entity(String name, String description)
  {
     // Name of a room or venue; nick for a person. 	
     this.name = name;
     // Description of a room or full name for a person.	
     this.description = description;	
  }

  public void setOrganization(String org) { organization = org; }

  public String getOrganization() { return organization; }  

  // Returns the string indicating availability.
  public String getAvail() { return avail; }

  // Set the availability.
  public void setAvail(String status) { avail = status; } 

  // Returns whether a favorite or not. 
  public boolean isFavorite() { return favorite; }

  // Set the favorite flag.
  public void setFavorite(boolean b) { favorite = b; } 

  // Returns true if this entity is already active; false otherwise.
  public boolean isActive() { return active; }

  // Returns the name of this entity.
  public String getName() { return name; }

  // Assign a name for this entity.
  public void setName(String n) { name = n; } 

  // Returns the description of this entity.
  public String getDescription() { return description; }

  // Returns a set representing the contents of this entity.
  public synchronized Set getContents() { return info; }

  // Activates the entity and return an object to perform the desired task. 
  public Object activateEntity() 
  {
      active = true; 
      return null;	
  }

  // Change the state of this entity to inactive.
  public void deactivateEntity() { active = false; }

  
  // Add an Entity to the set of info for this Entity.
  public synchronized void add(Entity e) 
  { 
	if (info == null)
	    info = new HashSet();
	if (e != null)	
	    info.add(e); 
  }

  // Remove an Entity from the set of info for this Entity.
  public synchronized void remove(Entity e) 
  { 
	if (info!=null && info.contains(e))
	    info.remove(e); 
  }

  // Remove all the elements from the set of info for this Entity.
  public synchronized void removeAll()
  {
	if (info != null)
	    info.clear();
  }

  // Gets the description of this entity as a StyledDocument.
  protected StyledDocument getDisplay() { return null; }

  public static StyledDocument getStyledDocument()
  {
      StyledDocument doc = new DefaultStyledDocument();
      Style defaultStyle = StyleContext.getDefaultStyleContext().
            getStyle(StyleContext.DEFAULT_STYLE);
      StyleConstants.setFontSize(defaultStyle, 16);
      Style regular = doc.addStyle("regular", defaultStyle);
      Style bold = doc.addStyle("bold", regular);
      StyleConstants.setBold(bold, true);
      StyleConstants.setFontSize(bold, 18);
      Style icon = doc.addStyle("icon", regular);	
      return doc;	
  } 

  // Necessary to displaying this object as when displaying a JTree node.	
  public String toString() { return getName(); }

  // Necessary to sort a Set of these objects, as when creating a TreeSet.  
  public int compareTo(Object o)
  {
      return toString().compareTo(o.toString());
  } 

  // Necessary for maintaining a Set w/o duplicates.
  public boolean equals(Entity e) { return e.name.equals(this.name); }

  public int hashCode() { return name.hashCode(); } 
}
